# coding=utf-8
"""
Inka Algorithmic Music
Creates fully arranged algorithmic instrumental music.
Copyright (C) 2018  Udo Wollschläger

This file defines all general midi instruments
"""

from inkamusic.const import T_BASS, T_CHOR, T_SOLO, T_HMNY, T_PERC
from inkamusic.const import R_BASS, R_LOW, R_MEDI, R_HIGH, R_FULL
from inkamusic.const import P_STICK, P_ACCENT, P_BASS, P_HIGH, P_LOW, P_RIDE, P_SNARE
import inkamusic.music_parameter as mp

# speed constants
# max number of tones per second
# averaged over one bar the number of tones per second may not exceed AVG_TONES_PER_SEC_PERCENT of this number
SPEED_BASS = 7
SPEED_BASS_NO_AUTODAMP = 5  # used for slow bass instruments (strings, tuba, ...)
SPEED_HMNY = 8
SPEED_LOW = 10
SPEED_MEDI = 14
SPEED_MEDI_NO_AUTODAMP = 5
SPEED_HIGH = 16
SPEED_FULL = 14

SPEED_P_BASS = 8
SPEED_P_STICK = 10
SPEED_P_SNARE = 10
SPEED_P_ACCENT = 6
SPEED_P_LOW = 8
SPEED_P_HIGH = 9
SPEED_P_RIDE = 10

GM_MELODY = [
    ['Grand Piano', 0, [0, 0], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Grand Piano', 1, [0, 0], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Grand Piano', 2, [0, 0], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Grand Piano', 3, [0, 0], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Grand Piano', 4, [0, 0], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Grand Piano', 5, [0, 0], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Grand Piano', 6, [0, 0], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Grand Piano', 7, [0, 0], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Grand Piano', 8, [0, 0], T_SOLO, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Grand Piano', 9, [0, 0], T_CHOR, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Bright Grand Piano', 10, [0, 1], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Bright Grand Piano', 11, [0, 1], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Bright Grand Piano', 12, [0, 1], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Bright Grand Piano', 13, [0, 1], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Bright Grand Piano', 14, [0, 1], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Bright Grand Piano', 15, [0, 1], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Bright Grand Piano', 16, [0, 1], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Bright Grand Piano', 17, [0, 1], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Bright Grand Piano', 18, [0, 1], T_SOLO, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Bright Grand Piano', 19, [0, 1], T_CHOR, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Electric Grand', 20, [0, 2], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Electric Grand', 21, [0, 2], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Electric Grand', 22, [0, 2], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Electric Grand', 23, [0, 2], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Electric Grand', 24, [0, 2], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Electric Grand', 25, [0, 2], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Electric Grand', 26, [0, 2], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Electric Grand', 27, [0, 2], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Electric Grand', 28, [0, 2], T_SOLO, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Electric Grand', 29, [0, 2], T_CHOR, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Honky-tonk Piano', 30, [0, 3], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Honky-tonk Piano', 31, [0, 3], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Honky-tonk Piano', 32, [0, 3], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Honky-tonk Piano', 33, [0, 3], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Honky-tonk Piano', 34, [0, 3], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Honky-tonk Piano', 35, [0, 3], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Honky-tonk Piano', 36, [0, 3], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Honky-tonk Piano', 37, [0, 3], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Honky-tonk Piano', 38, [0, 3], T_SOLO, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Honky-tonk Piano', 39, [0, 3], T_CHOR, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Electric Piano', 40, [0, 4], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Electric Piano', 41, [0, 4], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Electric Piano', 42, [0, 4], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Electric Piano', 43, [0, 4], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Electric Piano', 44, [0, 4], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Electric Piano', 45, [0, 4], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Electric Piano', 46, [0, 4], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Electric Piano', 47, [0, 4], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Electric Piano', 48, [0, 4], T_SOLO, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Electric Piano', 49, [0, 4], T_CHOR, R_FULL, SPEED_FULL, 29, 92, 1],
    ['FM Electric Piano', 50, [0, 5], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['FM Electric Piano', 51, [0, 5], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['FM Electric Piano', 52, [0, 5], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['FM Electric Piano', 53, [0, 5], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['FM Electric Piano', 54, [0, 5], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['FM Electric Piano', 55, [0, 5], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['FM Electric Piano', 56, [0, 5], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['FM Electric Piano', 57, [0, 5], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['FM Electric Piano', 58, [0, 5], T_SOLO, R_FULL, SPEED_FULL, 29, 92, 1],
    ['FM Electric Piano', 59, [0, 5], T_CHOR, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Harpsichord', 60, [0, 6], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Harpsichord', 61, [0, 6], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Harpsichord', 62, [0, 6], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Harpsichord', 63, [0, 6], T_SOLO, R_MEDI, SPEED_MEDI, 56, 80, 1],
    ['Harpsichord', 64, [0, 6], T_CHOR, R_MEDI, SPEED_MEDI, 56, 80, 1],
    ['Harpsichord', 65, [0, 6], T_HMNY, R_MEDI, SPEED_HMNY, 56, 80, 1],
    ['Harpsichord', 66, [0, 6], T_SOLO, R_FULL, SPEED_FULL, 29, 80, 1],
    ['Harpsichord', 67, [0, 6], T_CHOR, R_FULL, SPEED_FULL, 29, 80, 1],
    ['Detuned EP 1', 68, [8, 4], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Detuned EP 1', 69, [8, 4], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Detuned EP 1', 70, [8, 4], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Detuned EP 1', 71, [8, 4], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Detuned EP 1', 72, [8, 4], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Detuned EP 1', 73, [8, 4], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Detuned EP 1', 74, [8, 4], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Detuned EP 1', 75, [8, 4], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Detuned EP 1', 76, [8, 4], T_SOLO, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Detuned EP 1', 77, [8, 4], T_CHOR, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Detuned EP 2', 78, [8, 5], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Detuned EP 2', 79, [8, 5], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Detuned EP 2', 80, [8, 5], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Detuned EP 2', 81, [8, 5], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Detuned EP 2', 82, [8, 5], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Detuned EP 2', 83, [8, 5], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Detuned EP 2', 84, [8, 5], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 1],
    ['Detuned EP 2', 85, [8, 5], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Detuned EP 2', 86, [8, 5], T_SOLO, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Detuned EP 2', 87, [8, 5], T_CHOR, R_FULL, SPEED_FULL, 29, 92, 1],
    ['Coupled Harpsichord', 88, [8, 6], T_BASS, R_BASS, SPEED_BASS, 29, 48, 1],
    ['Coupled Harpsichord', 89, [8, 6], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Coupled Harpsichord', 90, [8, 6], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Coupled Harpsichord', 91, [8, 6], T_SOLO, R_MEDI, SPEED_MEDI, 56, 80, 1],
    ['Coupled Harpsichord', 92, [8, 6], T_CHOR, R_MEDI, SPEED_MEDI, 56, 80, 1],
    ['Coupled Harpsichord', 93, [8, 6], T_HMNY, R_MEDI, SPEED_HMNY, 56, 80, 1],
    ['Coupled Harpsichord', 94, [8, 6], T_SOLO, R_FULL, SPEED_FULL, 29, 80, 1],
    ['Coupled Harpsichord', 95, [8, 6], T_CHOR, R_FULL, SPEED_FULL, 29, 80, 1],
    ['Clavinet Hohner', 96, [0, 7], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Clavinet Hohner', 97, [0, 7], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Clavinet Hohner', 98, [0, 7], T_SOLO, R_MEDI, SPEED_MEDI, 56, 80, 1],
    ['Clavinet Hohner', 99, [0, 7], T_CHOR, R_MEDI, SPEED_MEDI, 56, 80, 1],
    ['Clavinet Hohner', 100, [0, 7], T_HMNY, R_MEDI, SPEED_HMNY, 56, 80, 1],
    ['Clavinet Hohner', 101, [0, 7], T_SOLO, R_FULL, SPEED_FULL, 36, 80, 1],
    ['Clavinet Hohner', 102, [0, 7], T_CHOR, R_FULL, SPEED_FULL, 36, 80, 1],
    ['Celesta soft', 103, [0, 8], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Celesta soft', 104, [0, 8], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Celesta soft', 105, [0, 8], T_SOLO, R_HIGH, SPEED_HIGH, 74, 94, 1],
    ['Celesta soft', 106, [0, 8], T_CHOR, R_HIGH, SPEED_HIGH, 74, 94, 1],
    ['Celesta soft', 107, [0, 8], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Celesta soft', 108, [0, 8], T_SOLO, R_FULL, SPEED_FULL, 48, 94, 1],
    ['Celesta soft', 109, [0, 8], T_CHOR, R_FULL, SPEED_FULL, 48, 94, 1],
    ['Glockenspiel', 110, [0, 9], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Glockenspiel', 111, [0, 9], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Glockenspiel', 112, [0, 9], T_SOLO, R_FULL, SPEED_FULL, 66, 96, 1],
    ['Glockenspiel', 113, [0, 9], T_CHOR, R_FULL, SPEED_FULL, 66, 96, 1],
    ['Music Box', 114, [0, 10], T_SOLO, R_HIGH, SPEED_HIGH, 74, 94, 1],
    ['Music Box', 115, [0, 10], T_CHOR, R_HIGH, SPEED_HIGH, 74, 94, 1],
    ['Music Box', 116, [0, 10], T_SOLO, R_FULL, SPEED_FULL, 64, 94, 1],
    ['Music Box', 117, [0, 10], T_CHOR, R_FULL, SPEED_FULL, 64, 94, 1],
    ['Vibraphone', 118, [0, 11], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Vibraphone', 119, [0, 11], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Vibraphone', 120, [0, 11], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Vibraphone', 121, [0, 11], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Vibraphone', 122, [0, 11], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Vibraphone', 123, [0, 11], T_SOLO, R_FULL, SPEED_FULL, 48, 96, 1],
    ['Vibraphone', 124, [0, 11], T_CHOR, R_FULL, SPEED_FULL, 48, 96, 1],
    ['Marimba', 125, [0, 12], T_SOLO, R_LOW, SPEED_LOW, 41, 64, 1],
    ['Marimba', 126, [0, 12], T_CHOR, R_LOW, SPEED_LOW, 41, 64, 1],
    ['Marimba', 127, [0, 12], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Marimba', 128, [0, 12], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Marimba', 129, [0, 12], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Marimba', 130, [0, 12], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Marimba', 131, [0, 12], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Marimba', 132, [0, 12], T_SOLO, R_FULL, SPEED_FULL, 41, 96, 1],
    ['Marimba', 133, [0, 12], T_CHOR, R_FULL, SPEED_FULL, 41, 96, 1],
    ['Xylophone', 134, [0, 13], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Xylophone', 135, [0, 13], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Xylophone', 136, [0, 13], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Xylophone', 137, [0, 13], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Xylophone', 138, [0, 13], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Xylophone', 139, [0, 13], T_SOLO, R_FULL, SPEED_FULL, 48, 96, 1],
    ['Xylophone', 140, [0, 13], T_CHOR, R_FULL, SPEED_FULL, 48, 96, 1],
    ['Tubular Bells', 141, [0, 14], T_SOLO, R_FULL, SPEED_FULL, 58, 76, 1],
    ['Tubular Bells', 142, [0, 14], T_CHOR, R_FULL, SPEED_FULL, 58, 76, 1],
    ['Dulcimer', 143, [0, 15], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Dulcimer', 144, [0, 15], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Dulcimer', 145, [0, 15], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Dulcimer', 146, [0, 15], T_SOLO, R_FULL, SPEED_FULL, 48, 86, 1],
    ['Dulcimer', 147, [0, 15], T_CHOR, R_FULL, SPEED_FULL, 48, 86, 1],
    ['Tinker Bell', 148, [0, 112], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Tinker Bell', 149, [0, 112], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Tinker Bell', 150, [0, 112], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Tinker Bell', 151, [0, 112], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Tinker Bell', 152, [0, 112], T_SOLO, R_HIGH, SPEED_HIGH, 74, 97, 1],
    ['Tinker Bell', 153, [0, 112], T_CHOR, R_HIGH, SPEED_HIGH, 74, 97, 1],
    ['Tinker Bell', 154, [0, 112], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Tinker Bell', 155, [0, 112], T_SOLO, R_FULL, SPEED_FULL, 36, 97, 1],
    ['Tinker Bell', 156, [0, 112], T_CHOR, R_FULL, SPEED_FULL, 36, 97, 1],
    ['Steel Drums', 157, [0, 114], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Steel Drums', 158, [0, 114], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Steel Drums', 159, [0, 114], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Steel Drums', 160, [0, 114], T_SOLO, R_FULL, SPEED_FULL, 48, 86, 1],
    ['Steel Drums', 161, [0, 114], T_CHOR, R_FULL, SPEED_FULL, 48, 86, 1],
    ['Melodic Tom', 162, [0, 117], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Melodic Tom', 163, [0, 117], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Melodic Tom', 164, [0, 117], T_SOLO, R_FULL, SPEED_FULL, 40, 75, 1],
    ['Melodic Tom', 165, [0, 117], T_CHOR, R_FULL, SPEED_FULL, 40, 75, 1],
    ['Melodic Tom 2', 166, [8, 117], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Melodic Tom 2', 167, [8, 117], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Melodic Tom 2', 168, [8, 117], T_SOLO, R_FULL, SPEED_FULL, 40, 75, 1],
    ['Melodic Tom 2', 169, [8, 117], T_CHOR, R_FULL, SPEED_FULL, 40, 75, 1],
    ['Synth Drum', 170, [0, 118], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Synth Drum', 171, [0, 118], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Synth Drum', 172, [0, 118], T_SOLO, R_FULL, SPEED_FULL, 40, 75, 1],
    ['Synth Drum', 173, [0, 118], T_CHOR, R_FULL, SPEED_FULL, 40, 75, 1],
    ['Drawbar Organ', 174, [0, 16], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 30, 48, 0],
    ['Drawbar Organ', 175, [0, 16], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Drawbar Organ', 176, [0, 16], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Drawbar Organ', 177, [0, 16], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Drawbar Organ', 178, [0, 16], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Drawbar Organ', 179, [0, 16], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Drawbar Organ', 180, [0, 16], T_SOLO, R_FULL, SPEED_FULL, 30, 88, 0],
    ['Drawbar Organ', 181, [0, 16], T_CHOR, R_FULL, SPEED_FULL, 30, 88, 0],
    ['Percussive Organ', 182, [0, 17], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 30, 48, 0],
    ['Percussive Organ', 183, [0, 17], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Percussive Organ', 184, [0, 17], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Percussive Organ', 185, [0, 17], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Percussive Organ', 186, [0, 17], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Percussive Organ', 187, [0, 17], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Percussive Organ', 188, [0, 17], T_SOLO, R_FULL, SPEED_FULL, 30, 88, 0],
    ['Percussive Organ', 189, [0, 17], T_CHOR, R_FULL, SPEED_FULL, 30, 88, 0],
    ['Rock Organ', 190, [0, 18], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 30, 48, 0],
    ['Rock Organ', 191, [0, 18], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Rock Organ', 192, [0, 18], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Rock Organ', 193, [0, 18], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Rock Organ', 194, [0, 18], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Rock Organ', 195, [0, 18], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Rock Organ', 196, [0, 18], T_SOLO, R_FULL, SPEED_FULL, 30, 88, 0],
    ['Rock Organ', 197, [0, 18], T_CHOR, R_FULL, SPEED_FULL, 30, 88, 0],
    ['Church Organ', 198, [0, 19], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 30, 48, 0],
    ['Church Organ', 199, [0, 19], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Church Organ', 200, [0, 19], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Church Organ', 201, [0, 19], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Church Organ', 202, [0, 19], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Church Organ', 203, [0, 19], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Church Organ', 204, [0, 19], T_SOLO, R_FULL, SPEED_FULL, 30, 88, 0],
    ['Church Organ', 205, [0, 19], T_CHOR, R_FULL, SPEED_FULL, 30, 88, 0],
    ['Reed Organ', 206, [0, 20], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Reed Organ', 207, [0, 20], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Reed Organ', 208, [0, 20], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Reed Organ', 209, [0, 20], T_SOLO, R_FULL, SPEED_FULL, 42, 90, 0],
    ['Reed Organ', 210, [0, 20], T_CHOR, R_FULL, SPEED_FULL, 42, 90, 0],
    ['Accordion', 211, [0, 21], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Accordion', 212, [0, 21], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Accordion', 213, [0, 21], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Accordion', 214, [0, 21], T_SOLO, R_FULL, SPEED_FULL, 48, 88, 0],
    ['Accordion', 215, [0, 21], T_CHOR, R_FULL, SPEED_FULL, 48, 88, 0],
    ['Harmonica', 216, [0, 22], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Harmonica', 217, [0, 22], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Harmonica', 218, [0, 22], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Harmonica', 219, [0, 22], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Harmonica', 220, [0, 22], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Harmonica', 221, [0, 22], T_SOLO, R_FULL, SPEED_FULL, 48, 96, 0],
    ['Harmonica', 222, [0, 22], T_CHOR, R_FULL, SPEED_FULL, 48, 96, 0],
    ['Bandoneon', 223, [0, 23], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Bandoneon', 224, [0, 23], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Bandoneon', 225, [0, 23], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Bandoneon', 226, [0, 23], T_SOLO, R_FULL, SPEED_FULL, 42, 84, 0],
    ['Bandoneon', 227, [0, 23], T_CHOR, R_FULL, SPEED_FULL, 42, 84, 0],
    ['Nylon String Guitar', 228, [0, 24], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Nylon String Guitar', 229, [0, 24], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Nylon String Guitar', 230, [0, 24], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Nylon String Guitar', 231, [0, 24], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Nylon String Guitar', 232, [0, 24], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Nylon String Guitar', 233, [0, 24], T_SOLO, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Nylon String Guitar', 234, [0, 24], T_CHOR, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Steel String Guitar', 235, [0, 25], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Steel String Guitar', 236, [0, 25], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Steel String Guitar', 237, [0, 25], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Steel String Guitar', 238, [0, 25], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Steel String Guitar', 239, [0, 25], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Steel String Guitar', 240, [0, 25], T_SOLO, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Steel String Guitar', 241, [0, 25], T_CHOR, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Jazz Guitar', 242, [0, 26], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Jazz Guitar', 243, [0, 26], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Jazz Guitar', 244, [0, 26], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Jazz Guitar', 245, [0, 26], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Jazz Guitar', 246, [0, 26], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Jazz Guitar', 247, [0, 26], T_SOLO, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Jazz Guitar', 248, [0, 26], T_CHOR, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Clean Guitar', 249, [0, 27], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Clean Guitar', 250, [0, 27], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Clean Guitar', 251, [0, 27], T_SOLO, R_FULL, SPEED_FULL, 40, 78, 1],
    ['Clean Guitar', 252, [0, 27], T_CHOR, R_FULL, SPEED_FULL, 40, 78, 1],
    ['Palm muted Guitar', 253, [0, 28], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Palm muted Guitar', 254, [0, 28], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Palm muted Guitar', 255, [0, 28], T_SOLO, R_FULL, SPEED_FULL, 40, 78, 1],
    ['Palm muted Guitar', 256, [0, 28], T_CHOR, R_FULL, SPEED_FULL, 40, 78, 1],
    ['Overdrive Guitar', 257, [0, 29], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Overdrive Guitar', 258, [0, 29], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Overdrive Guitar', 259, [0, 29], T_SOLO, R_FULL, SPEED_FULL, 40, 78, 0],
    ['Overdrive Guitar', 260, [0, 29], T_CHOR, R_FULL, SPEED_FULL, 40, 78, 0],
    ['Distortion Guitar', 261, [0, 30], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Distortion Guitar', 262, [0, 30], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Distortion Guitar', 263, [0, 30], T_SOLO, R_FULL, SPEED_FULL, 40, 78, 0],
    ['Distortion Guitar', 264, [0, 30], T_CHOR, R_FULL, SPEED_FULL, 40, 78, 0],
    ['Guitar Harmonics', 265, [0, 31], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Guitar Harmonics', 266, [0, 31], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Guitar Harmonics', 267, [0, 31], T_SOLO, R_FULL, SPEED_FULL, 40, 78, 0],
    ['Guitar Harmonics', 268, [0, 31], T_CHOR, R_FULL, SPEED_FULL, 40, 78, 0],
    ['Funk Guitar', 269, [8, 28], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Funk Guitar', 270, [8, 28], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Funk Guitar', 271, [8, 28], T_SOLO, R_FULL, SPEED_FULL, 40, 78, 1],
    ['Funk Guitar', 272, [8, 28], T_CHOR, R_FULL, SPEED_FULL, 40, 78, 1],
    ['Feedback Guitar', 273, [8, 30], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Feedback Guitar', 274, [8, 30], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Feedback Guitar', 275, [8, 30], T_SOLO, R_FULL, SPEED_FULL, 40, 78, 1],
    ['Feedback Guitar', 276, [8, 30], T_CHOR, R_FULL, SPEED_FULL, 40, 78, 1],
    ['Acoustic Bass', 277, [0, 32], T_BASS, R_BASS, SPEED_BASS, 28, 48, 1],
    ['Acoustic Bass', 278, [0, 32], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Acoustic Bass', 279, [0, 32], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Acoustic Bass', 280, [0, 32], T_SOLO, R_FULL, SPEED_FULL, 28, 67, 1],
    ['Acoustic Bass', 281, [0, 32], T_CHOR, R_FULL, SPEED_FULL, 28, 67, 1],
    ['Fingered Bass', 282, [0, 33], T_BASS, R_BASS, SPEED_BASS, 28, 48, 1],
    ['Fingered Bass', 283, [0, 33], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Fingered Bass', 284, [0, 33], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Fingered Bass', 285, [0, 33], T_SOLO, R_FULL, SPEED_FULL, 28, 65, 1],
    ['Fingered Bass', 286, [0, 33], T_CHOR, R_FULL, SPEED_FULL, 28, 65, 1],
    ['Picked Bass', 287, [0, 34], T_BASS, R_BASS, SPEED_BASS, 28, 48, 1],
    ['Picked Bass', 288, [0, 34], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Picked Bass', 289, [0, 34], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Picked Bass', 290, [0, 34], T_SOLO, R_FULL, SPEED_FULL, 28, 64, 1],
    ['Picked Bass', 291, [0, 34], T_CHOR, R_FULL, SPEED_FULL, 28, 64, 1],
    ['Fretless Bass', 292, [0, 35], T_BASS, R_BASS, SPEED_BASS, 28, 48, 1],
    ['Fretless Bass', 293, [0, 35], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Fretless Bass', 294, [0, 35], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Fretless Bass', 295, [0, 35], T_SOLO, R_FULL, SPEED_FULL, 28, 64, 1],
    ['Fretless Bass', 296, [0, 35], T_CHOR, R_FULL, SPEED_FULL, 28, 64, 1],
    ['Slap Bass', 297, [0, 36], T_BASS, R_BASS, SPEED_BASS, 28, 48, 1],
    ['Slap Bass', 298, [0, 36], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Slap Bass', 299, [0, 36], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Slap Bass', 300, [0, 36], T_SOLO, R_FULL, SPEED_FULL, 28, 64, 1],
    ['Slap Bass', 301, [0, 36], T_CHOR, R_FULL, SPEED_FULL, 28, 64, 1],
    ['Pop Bass', 302, [0, 37], T_BASS, R_BASS, SPEED_BASS, 28, 48, 1],
    ['Pop Bass', 303, [0, 37], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Pop Bass', 304, [0, 37], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Pop Bass', 305, [0, 37], T_SOLO, R_FULL, SPEED_FULL, 28, 64, 1],
    ['Pop Bass', 306, [0, 37], T_CHOR, R_FULL, SPEED_FULL, 28, 64, 1],
    ['Synth Bass 1', 307, [0, 38], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 28, 48, 0],
    ['Synth Bass 1', 308, [0, 38], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Bass 1', 309, [0, 38], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Bass 1', 310, [0, 38], T_SOLO, R_FULL, SPEED_FULL, 28, 74, 0],
    ['Synth Bass 1', 311, [0, 38], T_CHOR, R_FULL, SPEED_FULL, 28, 74, 0],
    ['Synth Bass 2', 312, [0, 39], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 28, 48, 0],
    ['Synth Bass 2', 313, [0, 39], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Bass 2', 314, [0, 39], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Bass 2', 315, [0, 39], T_SOLO, R_FULL, SPEED_FULL, 28, 74, 0],
    ['Synth Bass 2', 316, [0, 39], T_CHOR, R_FULL, SPEED_FULL, 28, 74, 0],
    ['Synth Bass 3', 317, [8, 38], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 28, 48, 0],
    ['Synth Bass 3', 318, [8, 38], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Bass 3', 319, [8, 38], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Bass 3', 320, [8, 38], T_SOLO, R_FULL, SPEED_FULL, 28, 74, 0],
    ['Synth Bass 3', 321, [8, 38], T_CHOR, R_FULL, SPEED_FULL, 28, 74, 0],
    ['Synth Bass 4', 322, [8, 39], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 28, 48, 0],
    ['Synth Bass 4', 323, [8, 39], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Bass 4', 324, [8, 39], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Bass 4', 325, [8, 39], T_SOLO, R_FULL, SPEED_FULL, 28, 74, 0],
    ['Synth Bass 4', 326, [8, 39], T_CHOR, R_FULL, SPEED_FULL, 28, 74, 0],
    ['Violin', 327, [0, 40], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Violin', 328, [0, 40], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Violin', 329, [0, 40], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Violin', 330, [0, 40], T_SOLO, R_FULL, SPEED_FULL, 55, 90, 0],
    ['Violin', 331, [0, 40], T_CHOR, R_FULL, SPEED_FULL, 55, 90, 0],
    ['Slow Violin', 332, [8, 40], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Slow Violin', 333, [8, 40], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Slow Violin', 334, [8, 40], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Slow Violin', 335, [8, 40], T_SOLO, R_FULL, SPEED_FULL, 55, 90, 0],
    ['Slow Violin', 336, [8, 40], T_CHOR, R_FULL, SPEED_FULL, 55, 90, 0],
    ['Viola', 337, [0, 41], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Viola', 338, [0, 41], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Viola', 339, [0, 41], T_SOLO, R_HIGH, SPEED_HIGH, 74, 93, 0],
    ['Viola', 340, [0, 41], T_CHOR, R_HIGH, SPEED_HIGH, 74, 93, 0],
    ['Viola', 341, [0, 41], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Viola', 342, [0, 41], T_SOLO, R_FULL, SPEED_FULL, 48, 93, 0],
    ['Viola', 343, [0, 41], T_CHOR, R_FULL, SPEED_FULL, 48, 93, 0],
    ['Cello', 344, [0, 42], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Cello', 345, [0, 42], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Cello', 346, [0, 42], T_SOLO, R_MEDI, SPEED_MEDI, 56, 80, 0],
    ['Cello', 347, [0, 42], T_CHOR, R_MEDI, SPEED_MEDI, 56, 80, 0],
    ['Cello', 348, [0, 42], T_HMNY, R_MEDI, SPEED_HMNY, 56, 80, 0],
    ['Cello', 349, [0, 42], T_SOLO, R_FULL, SPEED_FULL, 36, 80, 0],
    ['Cello', 350, [0, 42], T_CHOR, R_FULL, SPEED_FULL, 36, 80, 0],
    ['Contrabass', 351, [0, 43], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 28, 48, 0],
    ['Contrabass', 352, [0, 43], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Contrabass', 353, [0, 43], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Contrabass', 354, [0, 43], T_SOLO, R_FULL, SPEED_FULL, 28, 67, 0],
    ['Contrabass', 355, [0, 43], T_CHOR, R_FULL, SPEED_FULL, 28, 67, 0],
    ['Tremolo Strings', 356, [0, 44], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Tremolo Strings', 357, [0, 44], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Tremolo Strings', 358, [0, 44], T_SOLO, R_HIGH, SPEED_HIGH, 74, 94, 0],
    ['Tremolo Strings', 359, [0, 44], T_CHOR, R_HIGH, SPEED_HIGH, 74, 94, 0],
    ['Tremolo Strings', 360, [0, 44], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Tremolo Strings', 361, [0, 44], T_SOLO, R_FULL, SPEED_FULL, 55, 94, 0],
    ['Tremolo Strings', 362, [0, 44], T_CHOR, R_FULL, SPEED_FULL, 55, 94, 0],
    ['Pizzicato Strings', 363, [0, 45], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Pizzicato Strings', 364, [0, 45], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Pizzicato Strings', 365, [0, 45], T_SOLO, R_HIGH, SPEED_HIGH, 74, 94, 0],
    ['Pizzicato Strings', 366, [0, 45], T_CHOR, R_HIGH, SPEED_HIGH, 74, 94, 0],
    ['Pizzicato Strings', 367, [0, 45], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Pizzicato Strings', 368, [0, 45], T_SOLO, R_FULL, SPEED_FULL, 55, 94, 0],
    ['Pizzicato Strings', 369, [0, 45], T_CHOR, R_FULL, SPEED_FULL, 55, 94, 0],
    ['Orchestral Harp', 370, [0, 46], T_BASS, R_BASS, SPEED_BASS, 28, 48, 1],
    ['Orchestral Harp', 371, [0, 46], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Orchestral Harp', 372, [0, 46], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Orchestral Harp', 373, [0, 46], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Orchestral Harp', 374, [0, 46], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Orchestral Harp', 375, [0, 46], T_SOLO, R_HIGH, SPEED_HIGH, 74, 94, 1],
    ['Orchestral Harp', 376, [0, 46], T_CHOR, R_HIGH, SPEED_HIGH, 74, 94, 1],
    ['Orchestral Harp', 377, [0, 46], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Orchestral Harp', 378, [0, 46], T_SOLO, R_FULL, SPEED_FULL, 28, 94, 1],
    ['Orchestral Harp', 379, [0, 46], T_CHOR, R_FULL, SPEED_FULL, 28, 94, 1],
    ['Orchestral Pad', 380, [8, 48], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 28, 48, 0],
    ['Orchestral Pad', 381, [8, 48], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Orchestral Pad', 382, [8, 48], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Orchestral Pad', 383, [8, 48], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Orchestral Pad', 384, [8, 48], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Orchestral Pad', 385, [8, 48], T_SOLO, R_HIGH, SPEED_HIGH, 74, 94, 0],
    ['Orchestral Pad', 386, [8, 48], T_CHOR, R_HIGH, SPEED_HIGH, 74, 94, 0],
    ['Orchestral Pad', 387, [8, 48], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Orchestral Pad', 388, [8, 48], T_SOLO, R_FULL, SPEED_FULL, 28, 94, 0],
    ['Orchestral Pad', 389, [8, 48], T_CHOR, R_FULL, SPEED_FULL, 28, 94, 0],
    ['Strings', 390, [0, 48], T_SOLO, R_MEDI, SPEED_MEDI_NO_AUTODAMP, 56, 81, 0],
    ['Strings', 391, [0, 48], T_CHOR, R_MEDI, SPEED_MEDI_NO_AUTODAMP, 56, 81, 0],
    ['Strings', 392, [0, 48], T_HMNY, R_MEDI, SPEED_MEDI_NO_AUTODAMP, 56, 81, 0],
    ['Strings', 393, [0, 48], T_SOLO, R_FULL, SPEED_MEDI_NO_AUTODAMP, 55, 88, 0],
    ['Strings', 394, [0, 48], T_CHOR, R_FULL, SPEED_MEDI_NO_AUTODAMP, 55, 88, 0],
    ['Slow Strings', 395, [0, 49], T_SOLO, R_MEDI, SPEED_MEDI_NO_AUTODAMP, 56, 81, 0],
    ['Slow Strings', 396, [0, 49], T_CHOR, R_MEDI, SPEED_MEDI_NO_AUTODAMP, 56, 81, 0],
    ['Slow Strings', 397, [0, 49], T_HMNY, R_MEDI, SPEED_MEDI_NO_AUTODAMP, 56, 81, 0],
    ['Slow Strings', 398, [0, 49], T_SOLO, R_FULL, SPEED_MEDI_NO_AUTODAMP, 55, 88, 0],
    ['Slow Strings', 399, [0, 49], T_CHOR, R_FULL, SPEED_MEDI_NO_AUTODAMP, 55, 88, 0],
    ['Synth Strings 1', 400, [0, 50], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 30, 48, 0],
    ['Synth Strings 1', 401, [0, 50], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Strings 1', 402, [0, 50], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Strings 1', 403, [0, 50], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Strings 1', 404, [0, 50], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Strings 1', 405, [0, 50], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Synth Strings 1', 406, [0, 50], T_SOLO, R_FULL, SPEED_FULL, 30, 90, 0],
    ['Synth Strings 1', 407, [0, 50], T_CHOR, R_FULL, SPEED_FULL, 30, 90, 0],
    ['Synth Strings 2', 408, [0, 51], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 30, 48, 0],
    ['Synth Strings 2', 409, [0, 51], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Strings 2', 410, [0, 51], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Strings 2', 411, [0, 51], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Strings 2', 412, [0, 51], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Strings 2', 413, [0, 51], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Synth Strings 2', 414, [0, 51], T_SOLO, R_FULL, SPEED_FULL, 30, 90, 0],
    ['Synth Strings 2', 415, [0, 51], T_CHOR, R_FULL, SPEED_FULL, 30, 90, 0],
    ['Synth Strings 3', 416, [8, 50], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 30, 48, 0],
    ['Synth Strings 3', 417, [8, 50], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Strings 3', 418, [8, 50], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Synth Strings 3', 419, [8, 50], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Strings 3', 420, [8, 50], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Strings 3', 421, [8, 50], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Synth Strings 3', 422, [8, 50], T_SOLO, R_FULL, SPEED_FULL, 30, 90, 0],
    ['Synth Strings 3', 423, [8, 50], T_CHOR, R_FULL, SPEED_FULL, 30, 90, 0],
    ['Choir Aahs', 424, [0, 52], T_SOLO, R_FULL, SPEED_FULL, 56, 74, 0],
    ['Choir Aahs', 425, [0, 52], T_CHOR, R_FULL, SPEED_FULL, 56, 74, 0],
    ['Voice Oohs', 426, [0, 53], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Voice Oohs', 427, [0, 53], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Voice Oohs', 428, [0, 53], T_SOLO, R_MEDI, SPEED_MEDI, 56, 79, 0],
    ['Voice Oohs', 429, [0, 53], T_CHOR, R_MEDI, SPEED_MEDI, 56, 79, 0],
    ['Voice Oohs', 430, [0, 53], T_HMNY, R_MEDI, SPEED_HMNY, 56, 79, 0],
    ['Voice Oohs', 431, [0, 53], T_SOLO, R_FULL, SPEED_FULL, 40, 79, 0],
    ['Voice Oohs', 432, [0, 53], T_CHOR, R_FULL, SPEED_FULL, 40, 79, 0],
    ['Synth Choir', 433, [0, 54], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Choir', 434, [0, 54], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Choir', 435, [0, 54], T_SOLO, R_MEDI, SPEED_MEDI, 56, 79, 0],
    ['Synth Choir', 436, [0, 54], T_CHOR, R_MEDI, SPEED_MEDI, 56, 79, 0],
    ['Synth Choir', 437, [0, 54], T_HMNY, R_MEDI, SPEED_HMNY, 56, 79, 0],
    ['Synth Choir', 438, [0, 54], T_SOLO, R_FULL, SPEED_FULL, 40, 79, 0],
    ['Synth Choir', 439, [0, 54], T_CHOR, R_FULL, SPEED_FULL, 40, 79, 0],
    ['Trumpet', 440, [0, 56], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Trumpet', 441, [0, 56], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Trumpet', 442, [0, 56], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Trumpet', 443, [0, 56], T_SOLO, R_FULL, SPEED_FULL, 52, 83, 0],
    ['Trumpet', 444, [0, 56], T_CHOR, R_FULL, SPEED_FULL, 52, 83, 0],
    ['Trombone', 445, [0, 57], T_SOLO, R_FULL, SPEED_FULL, 42, 70, 0],
    ['Trombone', 446, [0, 57], T_CHOR, R_FULL, SPEED_FULL, 42, 70, 0],
    ['Tuba', 447, [0, 58], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 28, 48, 0],
    ['Tuba', 448, [0, 58], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Tuba', 449, [0, 58], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Tuba', 450, [0, 58], T_SOLO, R_FULL, SPEED_FULL, 28, 65, 0],
    ['Tuba', 451, [0, 58], T_CHOR, R_FULL, SPEED_FULL, 28, 65, 0],
    ['Muted Trumpet', 452, [0, 59], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Muted Trumpet', 453, [0, 59], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Muted Trumpet', 454, [0, 59], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Muted Trumpet', 455, [0, 59], T_SOLO, R_FULL, SPEED_FULL, 52, 82, 0],
    ['Muted Trumpet', 456, [0, 59], T_CHOR, R_FULL, SPEED_FULL, 52, 82, 0],
    ['French Horn', 457, [0, 60], T_SOLO, R_FULL, SPEED_FULL, 42, 74, 0],
    ['French Horn', 458, [0, 60], T_CHOR, R_FULL, SPEED_FULL, 42, 74, 0],
    ['Brass Section', 459, [0, 61], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Brass Section', 460, [0, 61], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Brass Section', 461, [0, 61], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Brass Section', 462, [0, 61], T_SOLO, R_FULL, SPEED_FULL, 50, 88, 0],
    ['Brass Section', 463, [0, 61], T_CHOR, R_FULL, SPEED_FULL, 50, 88, 0],
    ['Brass 2', 464, [8, 61], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Brass 2', 465, [8, 61], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Brass 2', 466, [8, 61], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Brass 2', 467, [8, 61], T_SOLO, R_FULL, SPEED_FULL, 50, 88, 0],
    ['Brass 2', 468, [8, 61], T_CHOR, R_FULL, SPEED_FULL, 50, 88, 0],
    ['Synth Brass 1', 469, [0, 62], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Brass 1', 470, [0, 62], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Brass 1', 471, [0, 62], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Brass 1', 472, [0, 62], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Brass 1', 473, [0, 62], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Synth Brass 1', 474, [0, 62], T_SOLO, R_FULL, SPEED_FULL, 40, 88, 0],
    ['Synth Brass 1', 475, [0, 62], T_CHOR, R_FULL, SPEED_FULL, 40, 88, 0],
    ['Synth Brass 2', 476, [0, 63], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Brass 2', 477, [0, 63], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Brass 2', 478, [0, 63], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Brass 2', 479, [0, 63], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Brass 2', 480, [0, 63], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Synth Brass 2', 481, [0, 63], T_SOLO, R_FULL, SPEED_FULL, 40, 88, 0],
    ['Synth Brass 2', 482, [0, 63], T_CHOR, R_FULL, SPEED_FULL, 40, 88, 0],
    ['Synth Brass 3', 483, [8, 62], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Brass 3', 484, [8, 62], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Brass 3', 485, [8, 62], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Brass 3', 486, [8, 62], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Brass 3', 487, [8, 62], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Synth Brass 3', 488, [8, 62], T_SOLO, R_FULL, SPEED_FULL, 40, 88, 0],
    ['Synth Brass 3', 489, [8, 62], T_CHOR, R_FULL, SPEED_FULL, 40, 88, 0],
    ['Synth Brass 4', 490, [8, 63], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Brass 4', 491, [8, 63], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 0],
    ['Synth Brass 4', 492, [8, 63], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Brass 4', 493, [8, 63], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Synth Brass 4', 494, [8, 63], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Synth Brass 4', 495, [8, 63], T_SOLO, R_FULL, SPEED_FULL, 40, 88, 0],
    ['Synth Brass 4', 496, [8, 63], T_CHOR, R_FULL, SPEED_FULL, 40, 88, 0],
    ['Soprano Sax', 497, [0, 64], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Soprano Sax', 498, [0, 64], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Soprano Sax', 499, [0, 64], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Soprano Sax', 500, [0, 64], T_SOLO, R_FULL, SPEED_FULL, 56, 88, 0],
    ['Soprano Sax', 501, [0, 64], T_CHOR, R_FULL, SPEED_FULL, 56, 88, 0],
    ['Alto Sax', 502, [0, 65], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Alto Sax', 503, [0, 65], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Alto Sax', 504, [0, 65], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Alto Sax', 505, [0, 65], T_SOLO, R_FULL, SPEED_FULL, 49, 87, 0],
    ['Alto Sax', 506, [0, 65], T_CHOR, R_FULL, SPEED_FULL, 49, 87, 0],
    ['Tenor Sax', 507, [0, 66], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Tenor Sax', 508, [0, 66], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Tenor Sax', 509, [0, 66], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Tenor Sax', 510, [0, 66], T_SOLO, R_FULL, SPEED_FULL, 44, 82, 0],
    ['Tenor Sax', 511, [0, 66], T_CHOR, R_FULL, SPEED_FULL, 44, 82, 0],
    ['Baritone Sax', 512, [0, 67], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Baritone Sax', 513, [0, 67], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Baritone Sax', 514, [0, 67], T_SOLO, R_FULL, SPEED_FULL, 38, 75, 0],
    ['Baritone Sax', 515, [0, 67], T_CHOR, R_FULL, SPEED_FULL, 38, 75, 0],
    ['Oboe', 516, [0, 68], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Oboe', 517, [0, 68], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Oboe', 518, [0, 68], T_SOLO, R_HIGH, SPEED_HIGH, 74, 93, 0],
    ['Oboe', 519, [0, 68], T_CHOR, R_HIGH, SPEED_HIGH, 74, 93, 0],
    ['Oboe', 520, [0, 68], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Oboe', 521, [0, 68], T_SOLO, R_FULL, SPEED_FULL, 56, 93, 0],
    ['Oboe', 522, [0, 68], T_CHOR, R_FULL, SPEED_FULL, 56, 93, 0],
    ['English Horn', 523, [0, 69], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['English Horn', 524, [0, 69], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['English Horn', 525, [0, 69], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['English Horn', 526, [0, 69], T_SOLO, R_FULL, SPEED_FULL, 52, 83, 0],
    ['English Horn', 527, [0, 69], T_CHOR, R_FULL, SPEED_FULL, 52, 83, 0],
    ['Bassoon', 528, [0, 70], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Bassoon', 529, [0, 70], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Bassoon', 530, [0, 70], T_SOLO, R_FULL, SPEED_FULL, 34, 76, 0],
    ['Bassoon', 531, [0, 70], T_CHOR, R_FULL, SPEED_FULL, 34, 76, 0],
    ['Clarinet', 532, [0, 71], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Clarinet', 533, [0, 71], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Clarinet', 534, [0, 71], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Clarinet', 535, [0, 71], T_SOLO, R_FULL, SPEED_FULL, 49, 83, 0],
    ['Clarinet', 536, [0, 71], T_CHOR, R_FULL, SPEED_FULL, 49, 83, 0],
    ['Piccolo', 537, [0, 72], T_SOLO, R_FULL, SPEED_FULL, 67, 90, 0],
    ['Piccolo', 538, [0, 72], T_CHOR, R_FULL, SPEED_FULL, 67, 90, 0],
    ['Flute', 539, [0, 73], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Flute', 540, [0, 73], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Flute', 541, [0, 73], T_SOLO, R_HIGH, SPEED_HIGH, 74, 98, 0],
    ['Flute', 542, [0, 73], T_CHOR, R_HIGH, SPEED_HIGH, 74, 98, 0],
    ['Flute', 543, [0, 73], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Flute', 544, [0, 73], T_SOLO, R_FULL, SPEED_FULL, 56, 98, 0],
    ['Flute', 545, [0, 73], T_CHOR, R_FULL, SPEED_FULL, 56, 98, 0],
    ['Recorder', 546, [0, 74], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Recorder', 547, [0, 74], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Recorder', 548, [0, 74], T_SOLO, R_FULL, SPEED_FULL, 72, 96, 0],
    ['Recorder', 549, [0, 74], T_CHOR, R_FULL, SPEED_FULL, 72, 96, 0],
    ['Pan Flute', 550, [0, 75], T_SOLO, R_MEDI, SPEED_MEDI, 58, 81, 0],
    ['Pan Flute', 551, [0, 75], T_CHOR, R_MEDI, SPEED_MEDI, 58, 81, 0],
    ['Pan Flute', 552, [0, 75], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Pan Flute', 553, [0, 75], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Pan Flute', 554, [0, 75], T_HMNY, R_MEDI, SPEED_HMNY, 58, 81, 0],
    ['Pan Flute', 555, [0, 75], T_SOLO, R_FULL, SPEED_FULL, 58, 96, 0],
    ['Pan Flute', 556, [0, 75], T_CHOR, R_FULL, SPEED_FULL, 58, 96, 0],
    ['Blown bottle', 557, [0, 76], T_SOLO, R_HIGH, SPEED_HIGH, 74, 91, 0],
    ['Blown bottle', 558, [0, 76], T_CHOR, R_HIGH, SPEED_HIGH, 74, 91, 0],
    ['Blown bottle', 559, [0, 76], T_SOLO, R_FULL, SPEED_FULL, 67, 91, 0],
    ['Blown bottle', 560, [0, 76], T_CHOR, R_FULL, SPEED_FULL, 67, 91, 0],
    ['Shakuhachi', 561, [0, 77], T_SOLO, R_HIGH, SPEED_HIGH, 74, 91, 0],
    ['Shakuhachi', 562, [0, 77], T_CHOR, R_HIGH, SPEED_HIGH, 74, 91, 0],
    ['Shakuhachi', 563, [0, 77], T_SOLO, R_FULL, SPEED_FULL, 67, 91, 0],
    ['Shakuhachi', 564, [0, 77], T_CHOR, R_FULL, SPEED_FULL, 67, 91, 0],
    ['Whistle', 565, [0, 78], T_SOLO, R_HIGH, SPEED_HIGH, 74, 92, 0],
    ['Whistle', 566, [0, 78], T_CHOR, R_HIGH, SPEED_HIGH, 74, 92, 0],
    ['Whistle', 567, [0, 78], T_SOLO, R_FULL, SPEED_FULL, 74, 92, 0],
    ['Whistle', 568, [0, 78], T_CHOR, R_FULL, SPEED_FULL, 74, 92, 0],
    ['Ocarina', 569, [0, 79], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Ocarina', 570, [0, 79], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Ocarina', 571, [0, 79], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Ocarina', 572, [0, 79], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Ocarina', 573, [0, 79], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Ocarina', 574, [0, 79], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Ocarina', 575, [0, 79], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Ocarina', 576, [0, 79], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Ocarina', 577, [0, 79], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Square Lead', 578, [0, 80], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Square Lead', 579, [0, 80], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Square Lead', 580, [0, 80], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Square Lead', 581, [0, 80], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Square Lead', 582, [0, 80], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Square Lead', 583, [0, 80], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Square Lead', 584, [0, 80], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Square Lead', 585, [0, 80], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Square Lead', 586, [0, 80], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Saw Wave', 587, [0, 81], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Saw Wave', 588, [0, 81], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Saw Wave', 589, [0, 81], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Saw Wave', 590, [0, 81], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Saw Wave', 591, [0, 81], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Saw Wave', 592, [0, 81], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Saw Wave', 593, [0, 81], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Saw Wave', 594, [0, 81], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Saw Wave', 595, [0, 81], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Calliope', 596, [0, 82], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Calliope', 597, [0, 82], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Calliope', 598, [0, 82], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Calliope', 599, [0, 82], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Calliope', 600, [0, 82], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Calliope', 601, [0, 82], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Calliope', 602, [0, 82], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Calliope', 603, [0, 82], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Calliope', 604, [0, 82], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Chiffer', 605, [0, 83], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Chiffer', 606, [0, 83], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Chiffer', 607, [0, 83], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Chiffer', 608, [0, 83], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Chiffer', 609, [0, 83], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Chiffer', 610, [0, 83], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Chiffer', 611, [0, 83], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Chiffer', 612, [0, 83], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Chiffer', 613, [0, 83], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Charang', 614, [0, 84], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Charang', 615, [0, 84], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Charang', 616, [0, 84], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Charang', 617, [0, 84], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Charang', 618, [0, 84], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Charang', 619, [0, 84], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Charang', 620, [0, 84], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Charang', 621, [0, 84], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Charang', 622, [0, 84], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Solo Vox', 623, [0, 85], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Solo Vox', 624, [0, 85], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Solo Vox', 625, [0, 85], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Solo Vox', 626, [0, 85], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Solo Vox', 627, [0, 85], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Solo Vox', 628, [0, 85], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Solo Vox', 629, [0, 85], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Solo Vox', 630, [0, 85], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Solo Vox', 631, [0, 85], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['5th Sawtooth Wave', 632, [0, 86], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['5th Sawtooth Wave', 633, [0, 86], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['5th Sawtooth Wave', 634, [0, 86], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['5th Sawtooth Wave', 635, [0, 86], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['5th Sawtooth Wave', 636, [0, 86], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['5th Sawtooth Wave', 637, [0, 86], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['5th Sawtooth Wave', 638, [0, 86], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['5th Sawtooth Wave', 639, [0, 86], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['5th Sawtooth Wave', 640, [0, 86], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Bass and Lead', 641, [0, 87], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Bass and Lead', 642, [0, 87], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Bass and Lead', 643, [0, 87], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Bass and Lead', 644, [0, 87], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Bass and Lead', 645, [0, 87], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Bass and Lead', 646, [0, 87], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Bass and Lead', 647, [0, 87], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Bass and Lead', 648, [0, 87], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Bass and Lead', 649, [0, 87], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Fantasia', 650, [0, 88], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Fantasia', 651, [0, 88], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Fantasia', 652, [0, 88], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Fantasia', 653, [0, 88], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Fantasia', 654, [0, 88], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Fantasia', 655, [0, 88], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Fantasia', 656, [0, 88], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Fantasia', 657, [0, 88], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Fantasia', 658, [0, 88], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Warm Pad', 659, [0, 89], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Warm Pad', 660, [0, 89], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Warm Pad', 661, [0, 89], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Warm Pad', 662, [0, 89], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Warm Pad', 663, [0, 89], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Warm Pad', 664, [0, 89], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Warm Pad', 665, [0, 89], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Warm Pad', 666, [0, 89], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Warm Pad', 667, [0, 89], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Polysynth', 668, [0, 90], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Polysynth', 669, [0, 90], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Polysynth', 670, [0, 90], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Polysynth', 671, [0, 90], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Polysynth', 672, [0, 90], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Polysynth', 673, [0, 90], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Polysynth', 674, [0, 90], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Polysynth', 675, [0, 90], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Polysynth', 676, [0, 90], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Space Voice', 677, [0, 91], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Space Voice', 678, [0, 91], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Space Voice', 679, [0, 91], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Space Voice', 680, [0, 91], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Space Voice', 681, [0, 91], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Space Voice', 682, [0, 91], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Space Voice', 683, [0, 91], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Space Voice', 684, [0, 91], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Space Voice', 685, [0, 91], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Bowed Glass', 686, [0, 92], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Bowed Glass', 687, [0, 92], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Bowed Glass', 688, [0, 92], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Bowed Glass', 689, [0, 92], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Bowed Glass', 690, [0, 92], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Bowed Glass', 691, [0, 92], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Bowed Glass', 692, [0, 92], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Bowed Glass', 693, [0, 92], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Bowed Glass', 694, [0, 92], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Metal Pad', 695, [0, 93], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Metal Pad', 696, [0, 93], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Metal Pad', 697, [0, 93], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Metal Pad', 698, [0, 93], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Metal Pad', 699, [0, 93], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Metal Pad', 700, [0, 93], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Metal Pad', 701, [0, 93], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Metal Pad', 702, [0, 93], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Metal Pad', 703, [0, 93], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Halo Pad', 704, [0, 94], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Halo Pad', 705, [0, 94], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Halo Pad', 706, [0, 94], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Halo Pad', 707, [0, 94], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Halo Pad', 708, [0, 94], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Halo Pad', 709, [0, 94], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Halo Pad', 710, [0, 94], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Halo Pad', 711, [0, 94], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Halo Pad', 712, [0, 94], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Sweep Pad', 713, [0, 95], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Sweep Pad', 714, [0, 95], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Sweep Pad', 715, [0, 95], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Sweep Pad', 716, [0, 95], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Sweep Pad', 717, [0, 95], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Sweep Pad', 718, [0, 95], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Sweep Pad', 719, [0, 95], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Sweep Pad', 720, [0, 95], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Sweep Pad', 721, [0, 95], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Ice Rain', 722, [0, 96], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Ice Rain', 723, [0, 96], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Ice Rain', 724, [0, 96], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Ice Rain', 725, [0, 96], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Ice Rain', 726, [0, 96], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Ice Rain', 727, [0, 96], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Ice Rain', 728, [0, 96], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Ice Rain', 729, [0, 96], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Ice Rain', 730, [0, 96], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Sound Track', 731, [0, 97], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Sound Track', 732, [0, 97], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Sound Track', 733, [0, 97], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Sound Track', 734, [0, 97], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Sound Track', 735, [0, 97], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Sound Track', 736, [0, 97], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Sound Track', 737, [0, 97], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Sound Track', 738, [0, 97], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Sound Track', 739, [0, 97], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Crystal', 740, [0, 98], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Crystal', 741, [0, 98], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Crystal', 742, [0, 98], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Crystal', 743, [0, 98], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Crystal', 744, [0, 98], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Crystal', 745, [0, 98], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Crystal', 746, [0, 98], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Crystal', 747, [0, 98], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Crystal', 748, [0, 98], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Atmosphere', 749, [0, 99], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Atmosphere', 750, [0, 99], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Atmosphere', 751, [0, 99], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Atmosphere', 752, [0, 99], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Atmosphere', 753, [0, 99], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Atmosphere', 754, [0, 99], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Atmosphere', 755, [0, 99], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Atmosphere', 756, [0, 99], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Atmosphere', 757, [0, 99], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Brightness', 758, [0, 100], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Brightness', 759, [0, 100], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Brightness', 760, [0, 100], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Brightness', 761, [0, 100], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Brightness', 762, [0, 100], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Brightness', 763, [0, 100], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Brightness', 764, [0, 100], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Brightness', 765, [0, 100], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Brightness', 766, [0, 100], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Goblin', 767, [0, 101], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Goblin', 768, [0, 101], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Goblin', 769, [0, 101], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Goblin', 770, [0, 101], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Goblin', 771, [0, 101], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Goblin', 772, [0, 101], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Goblin', 773, [0, 101], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Goblin', 774, [0, 101], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Goblin', 775, [0, 101], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Echo Drops', 776, [0, 102], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Echo Drops', 777, [0, 102], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Echo Drops', 778, [0, 102], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Echo Drops', 779, [0, 102], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Echo Drops', 780, [0, 102], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Echo Drops', 781, [0, 102], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Echo Drops', 782, [0, 102], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Echo Drops', 783, [0, 102], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Echo Drops', 784, [0, 102], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Star Theme', 785, [0, 103], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Star Theme', 786, [0, 103], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Star Theme', 787, [0, 103], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Star Theme', 788, [0, 103], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Star Theme', 789, [0, 103], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Star Theme', 790, [0, 103], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Star Theme', 791, [0, 103], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Star Theme', 792, [0, 103], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Star Theme', 793, [0, 103], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 0],
    ['Sitar', 794, [0, 104], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Sitar', 795, [0, 104], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Sitar', 796, [0, 104], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Sitar', 797, [0, 104], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Sitar', 798, [0, 104], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Sitar', 799, [0, 104], T_SOLO, R_FULL, SPEED_FULL, 36, 84, 1],
    ['Sitar', 800, [0, 104], T_CHOR, R_FULL, SPEED_FULL, 36, 84, 1],
    ['Banjo', 801, [0, 105], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Banjo', 802, [0, 105], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Banjo', 803, [0, 105], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Banjo', 804, [0, 105], T_SOLO, R_FULL, SPEED_FULL, 48, 87, 1],
    ['Banjo', 805, [0, 105], T_CHOR, R_FULL, SPEED_FULL, 48, 87, 1],
    ['Shamisen', 806, [0, 106], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Shamisen', 807, [0, 106], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Shamisen', 808, [0, 106], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Shamisen', 809, [0, 106], T_SOLO, R_FULL, SPEED_FULL, 50, 83, 1],
    ['Shamisen', 810, [0, 106], T_CHOR, R_FULL, SPEED_FULL, 50, 83, 1],
    ['Koto', 811, [8, 107], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Koto', 812, [8, 107], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Koto', 813, [8, 107], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Koto', 814, [8, 107], T_SOLO, R_FULL, SPEED_FULL, 42, 83, 1],
    ['Koto', 815, [8, 107], T_CHOR, R_FULL, SPEED_FULL, 42, 83, 1],
    ['Taisho Koto', 816, [0, 107], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Taisho Koto', 817, [0, 107], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Taisho Koto', 818, [0, 107], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Taisho Koto', 819, [0, 107], T_SOLO, R_FULL, SPEED_FULL, 42, 83, 1],
    ['Taisho Koto', 820, [0, 107], T_CHOR, R_FULL, SPEED_FULL, 42, 83, 1],
    ['Kalimba', 821, [0, 108], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Kalimba', 822, [0, 108], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 1],
    ['Kalimba', 823, [0, 108], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Kalimba', 824, [0, 108], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Kalimba', 825, [0, 108], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Kalimba', 826, [0, 108], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 1],
    ['Kalimba', 827, [0, 108], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Kalimba', 828, [0, 108], T_SOLO, R_FULL, SPEED_FULL, 36, 96, 1],
    ['Kalimba', 829, [0, 108], T_CHOR, R_FULL, SPEED_FULL, 36, 96, 1],
    ['Fiddle', 830, [0, 110], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Fiddle', 831, [0, 110], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Fiddle', 832, [0, 110], T_SOLO, R_HIGH, SPEED_HIGH, 74, 94, 0],
    ['Fiddle', 833, [0, 110], T_CHOR, R_HIGH, SPEED_HIGH, 74, 94, 0],
    ['Fiddle', 834, [0, 110], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Fiddle', 835, [0, 110], T_SOLO, R_FULL, SPEED_FULL, 55, 94, 0],
    ['Fiddle', 836, [0, 110], T_CHOR, R_FULL, SPEED_FULL, 55, 94, 0],
    ['Italian Accordion', 837, [8, 21], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Italian Accordion', 838, [8, 21], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Italian Accordion', 839, [8, 21], T_SOLO, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Italian Accordion', 840, [8, 21], T_CHOR, R_HIGH, SPEED_HIGH, 74, 96, 0],
    ['Italian Accordion', 841, [8, 21], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Italian Accordion', 842, [8, 21], T_SOLO, R_FULL, SPEED_FULL, 54, 96, 0],
    ['Italian Accordion', 843, [8, 21], T_CHOR, R_FULL, SPEED_FULL, 54, 96, 0],
    ['Sine', 844, [8, 80], T_BASS, R_BASS, SPEED_BASS_NO_AUTODAMP, 28, 48, 0],
    ['Sine', 845, [8, 80], T_SOLO, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Sine', 846, [8, 80], T_CHOR, R_LOW, SPEED_LOW, 39, 64, 0],
    ['Sine', 847, [8, 80], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Sine', 848, [8, 80], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 0],
    ['Sine', 849, [8, 80], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 0],
    ['Sine', 850, [8, 80], T_SOLO, R_FULL, SPEED_FULL, 28, 88, 0],
    ['Sine', 851, [8, 80], T_CHOR, R_FULL, SPEED_FULL, 28, 88, 0],
    ['Ukulele', 852, [8, 24], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Ukulele', 853, [8, 24], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Ukulele', 854, [8, 24], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Ukulele', 855, [8, 24], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Ukulele', 856, [8, 24], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Ukulele', 857, [8, 24], T_SOLO, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Ukulele', 858, [8, 24], T_CHOR, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Mandolin', 859, [16, 25], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Mandolin', 860, [16, 25], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Mandolin', 861, [16, 25], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Mandolin', 862, [16, 25], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Mandolin', 863, [16, 25], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Mandolin', 864, [16, 25], T_SOLO, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Mandolin', 865, [16, 25], T_CHOR, R_FULL, SPEED_FULL, 40, 83, 1],
    ['12 String Guitar', 866, [8, 25], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['12 String Guitar', 867, [8, 25], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['12 String Guitar', 868, [8, 25], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['12 String Guitar', 869, [8, 25], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['12 String Guitar', 870, [8, 25], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['12 String Guitar', 871, [8, 25], T_SOLO, R_FULL, SPEED_FULL, 40, 83, 1],
    ['12 String Guitar', 872, [8, 25], T_CHOR, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Hawaiian Guitar', 873, [8, 26], T_SOLO, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Hawaiian Guitar', 874, [8, 26], T_CHOR, R_LOW, SPEED_LOW, 40, 64, 1],
    ['Hawaiian Guitar', 875, [8, 26], T_SOLO, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Hawaiian Guitar', 876, [8, 26], T_CHOR, R_MEDI, SPEED_MEDI, 56, 81, 1],
    ['Hawaiian Guitar', 877, [8, 26], T_HMNY, R_MEDI, SPEED_HMNY, 56, 81, 1],
    ['Hawaiian Guitar', 878, [8, 26], T_SOLO, R_FULL, SPEED_FULL, 40, 83, 1],
    ['Hawaiian Guitar', 879, [8, 26], T_CHOR, R_FULL, SPEED_FULL, 40, 83, 1],
]

GM_PERCUSSION = [
    ['Acoustic Bass Drum', 725, [0, -35], T_PERC, P_BASS, SPEED_P_BASS, 0, 0, 1],
    ['Bass Drum 1', 726, [0, -36], T_PERC, P_BASS, SPEED_P_BASS, 0, 0, 1],
    ['Side Stick', 729, [0, -37], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Acoustic Snare', 731, [0, -38], T_PERC, P_SNARE, SPEED_P_SNARE, 0, 0, 1],
    ['Electric Snare', 733, [0, -40], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Hand Clap', 734, [0, -39], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['High Floor Tom', 735, [0, -43], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['Low Floor Tom', 736, [0, -41], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['Low Mid Tom', 737, [0, -47], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['Low Tom', 738, [0, -45], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['High Tom', 739, [0, -50], T_PERC, P_HIGH, SPEED_P_HIGH, 0, 0, 1],
    ['High Mid Tom', 740, [0, -48], T_PERC, P_HIGH, SPEED_P_HIGH, 0, 0, 1],
    ['Closed Hi-hat', 741, [0, -42], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Pedal Hi-hat', 742, [0, -44], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Open Hi-hat', 743, [0, -46], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Crash Cymbal 1', 744, [0, -49], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Crash Cymbal 2', 745, [0, -57], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Ride Cymbal 1', 746, [0, -51], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['Ride Cymbal 2', 747, [0, -59], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['Chinese Cymbal', 748, [0, -52], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Splash Cymbal', 749, [0, -55], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Ride Bell', 750, [0, -53], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Cow Bell', 751, [0, -56], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Tambourine', 752, [0, -54], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['Vibra Slap', 753, [0, -58], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Low Bongo', 754, [0, -61], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['High Bongo', 755, [0, -60], T_PERC, P_HIGH, SPEED_P_HIGH, 0, 0, 1],
    ['Low Conga', 756, [0, -64], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['Mute High Conga', 757, [0, -62], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Open High Conga', 758, [0, -63], T_PERC, P_HIGH, SPEED_P_HIGH, 0, 0, 1],
    ['Low Timbale', 759, [0, -66], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['High Timbale', 760, [0, -65], T_PERC, P_HIGH, SPEED_P_HIGH, 0, 0, 1],
    ['Low Agogô', 761, [0, -68], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['High Agogô', 762, [0, -67], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['Cabasa', 763, [0, -69], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['Maracas', 764, [0, -70], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['Short Güiro', 765, [0, -73], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Long Güiro', 766, [0, -74], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Claves', 767, [0, -75], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Low Wood Block', 768, [0, -77], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['High Wood Block', 769, [0, -76], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Mute Cuica', 770, [0, -78], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Open Cuica', 771, [0, -79], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Mute Triangle', 772, [0, -80], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Open Triangle', 773, [0, -81], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],

    ]

LOGICPRO_PERCUSSION = [
    ['Acoustic Bass Drum', 725, [0, -35], T_PERC, P_BASS, SPEED_P_BASS, 0, 0, 1],
    ['Bass Drum 1', 726, [0, -36], T_PERC, P_BASS, SPEED_P_BASS, 0, 0, 1],
    ['Acoustic Snare', 731, [0, -38], T_PERC, P_SNARE, SPEED_P_SNARE, 0, 0, 1],
    ['Electric Snare', 733, [0, -40], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Hand Clap', 734, [0, -39], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['High Floor Tom', 735, [0, -43], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['Low Floor Tom', 736, [0, -41], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['Low Mid Tom', 737, [0, -47], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['Low Tom', 738, [0, -45], T_PERC, P_LOW, SPEED_P_LOW, 0, 0, 1],
    ['High Tom', 739, [0, -50], T_PERC, P_HIGH, SPEED_P_HIGH, 0, 0, 1],
    ['High Mid Tom', 740, [0, -48], T_PERC, P_HIGH, SPEED_P_HIGH, 0, 0, 1],
    ['Closed Hi-hat', 741, [0, -42], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Pedal Hi-hat', 742, [0, -44], T_PERC, P_STICK, SPEED_P_STICK, 0, 0, 1],
    ['Open Hi-hat', 743, [0, -46], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Crash Cymbal 1', 744, [0, -49], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Crash Cymbal 2', 745, [0, -57], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Ride Cymbal 1', 746, [0, -51], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['Ride Cymbal 2', 747, [0, -59], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],
    ['Chinese Cymbal', 748, [0, -52], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Ride Bell', 750, [0, -53], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Cow Bell', 751, [0, -56], T_PERC, P_ACCENT, SPEED_P_ACCENT, 0, 0, 1],
    ['Tambourine', 752, [0, -54], T_PERC, P_RIDE, SPEED_P_RIDE, 0, 0, 1],


    ]
if mp.LOGIC_PRO_MIDI_SETTINGS:
    GM_INSTRUMENTS = GM_MELODY + LOGICPRO_PERCUSSION  # GM_PERCUSSION
else:
    GM_INSTRUMENTS = GM_MELODY + GM_PERCUSSION
